﻿/*
* @author Antti
*/
package {
	
	import flash.display.*;
	import flash.events.*;
	import flash.utils.*;
	import flash.xml.*;
	import flash.net.*;
	import flash.text.*;
	import flash.system.*;
	
	import mx.effects.Tween;
	import mx.effects.easing.*;
	
	import org.papervision3d.scenes.*;
	import org.papervision3d.cameras.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.core.proto.*;
	
	import gfx.Form;
	import gfx.scene3d.*;

	public class Main extends Sprite {
		
		// Constants
		
		static public const AUTHOR		: String = "Antti Kupila";
		static public const AUTHOR_URL	: String = "http://www.anttikupila.com";
		static public const TITLE		: String = "flame";
		static public const VERSION		: String = "0.1.2";
		
		// Camera start positions
		protected var targetR				: Number = 0;
		protected var targetY				: Number = 100;
		protected var targetZ				: Number = -500;
		protected var zDelay				: Number = 25; // z pos
		protected var yDelay				: Number = 12; // y pos
		protected var rDelay				: Number = 15; // rotation
		protected var targetI				: int; // target clip
		
		// Vars
		
		static protected var console		: TextField;
		protected var form				: Form;
		protected var background			: Shape;
		protected var xmlLoader			: URLLoader;
		protected var artist				: String;
		
		protected var holder				: Sprite;
		protected var scene				: Scene3D;
		protected var rootNode			: DisplayObject3D;
		protected var camera				: Camera3D;
		protected var arcs				: Array = new Array( );
		protected var currentArc			: Arc;
		protected var isCloseUp			: Boolean;
		
		protected var backgroundTween	: Tween;
		
		[Embed(source="/fonts/MyriadWebPro.ttf", fontName="myriad", mimeType="application/x-font")]
		protected var myriad				: Class;
		
		static protected var instance		: Main;
		
		public function Main() {
			
			instance = this;
			
			stage.scaleMode = "noScale";
			stage.align = "LT";
			
			initBackground( );
			
			form = new Form( );
			
			addChild( background );
			addChild( form );
			
			init3DScene( );
			initListeners( );
			initTweens( );
			initLoaders( );
			
			resizeHandler( null ); // Center
			
			Security.loadPolicyFile("http://static.last.fm/crossdomain.xml");
		}
		
		// _________________________________________________________________________
		// PROTECTED METHODS
		
		protected function initBackground( ) : void {
			background = new Shape( );
			background.graphics.beginFill( 0xffffff, 100 );
			background.graphics.drawRect( 0, 0, stage.stageWidth, stage.stageHeight );
			background.graphics.endFill( );
		}
		
		protected function init3DScene( ) : void {
			holder = new Sprite( );
			addChild( holder );
			
			scene = new Scene3D( holder );
			
			rootNode = new DisplayObject3D( "rootNode" );
			scene.addChild( rootNode );
			
			camera = new Camera3D( );
			camera.zoom = 3.5;
			camera.z = -20000;
			camera.y = 7000;
			camera.lookAt( rootNode );
		}
		
		protected function initListeners( ) : void {	
			form.addEventListener( Form.SUBMIT, formSubmitHandler );
			stage.addEventListener( Event.RESIZE, resizeHandler );
			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyHandler );
		}
		
		protected function initTweens( ) : void {
			backgroundTween = new Tween( this, 100, 0, 2000, 31 );
			backgroundTween.setTweenHandlers( backgroundTweenHandler, init3D );
			backgroundTween.easingFunction = Quartic.easeOut;
			backgroundTween.pause( );
		}
		
		protected function initLoaders( ) : void {
			xmlLoader = new URLLoader( );
			xmlLoader.addEventListener( Event.COMPLETE, xmlLoaded );
		}
		
		protected function init3D( newPos : Number ) : void {
			form.visible = false;
			xmlLoader.load( new URLRequest( "http://ws.audioscrobbler.com/1.0/artist/" + artist + "/similar.xml" ) );
			this.addEventListener( Event.ENTER_FRAME, draw3D );
		}
		
		protected function draw3D( e : Event ) : void {
			var tZ : Number = targetZ;
			var tY : Number = targetY;
			if (isCloseUp == true) {
				tZ = -15;
				tY = 4;
			}
			camera.z += (tZ - camera.z) / zDelay;
			camera.y += (tY - camera.y) / yDelay;
			currentArc.rotationY += (targetR - currentArc.rotationY) / rDelay;
			scene.renderCamera( camera );
		}
		
		protected function updateRot( ) : void {
			targetR = (targetI / (currentArc.count)) * Arc.DEGREES - 90;
		}
		
		protected function doCloseUp( ) : void {
			currentArc.preview( targetI );
			isCloseUp = true;
		}
		
		protected function closeCloseUp( ) : void {
			currentArc.stopPreview( targetI );
			isCloseUp = false;
		}
		
		// _________________________________________________________________________
		// EVENT HANDLERS
		
		protected function resizeHandler( e : Event ) : void {
			form.x = stage.stageWidth/2 - form.width/2;
			form.y = stage.stageHeight/2 - form.height/2;
			background.width = stage.stageWidth;
			background.height = stage.stageHeight;
			holder.x = stage.stageWidth * 0.5;
			holder.y = stage.stageHeight * (2/3);
		}
		
		protected function formSubmitHandler( e : Event ) : void {
			if ( form.artist.length <= 0 || form.artist == "Artist name" ) return;
			artist = form.artist;
			backgroundTween.resume( );
		}
		
		protected function backgroundTweenHandler( newPos : Number ) : void {
			form.alpha = Math.pow( newPos/10, 2 ); // exponential tween
			form.enabled = (newPos >= 100);
			background.alpha = newPos;
		}
		
		protected function xmlLoaded( e : Event ) : void {
			var arc : Arc = new Arc( new XML( xmlLoader.data ), arcs.length, artist );
				arc.addEventListener( Event.CHANGE, arcChangeHandler );
			arcs.push( arc );
			rootNode.addChild( arc );
			currentArc = arc;
		}
		
		protected function arcChangeHandler( e: Event ) : void {
			targetI = Math.floor( e.target.count / 2 );
			updateRot();
		}
		
		protected function keyHandler( e : KeyboardEvent ) : void {
			if ( artist == null ) {
				if ( e.keyCode == 13 ) { // Enter
					formSubmitHandler( null );
				}
			} else {
				switch ( e.keyCode ) {
					case 37: // Left
						if (isCloseUp == true) currentArc.stopPreview( targetI );
						targetI--;
						if ( targetI < 0 ) targetI = 0;
						if (isCloseUp == true) currentArc.preview( targetI );
						updateRot( );
						break;
					case 39: // Right
						if (isCloseUp == true) currentArc.stopPreview( targetI );
						targetI++;
						if ( targetI >= currentArc.count-1 ) targetI = currentArc.count-1;
						if (isCloseUp == true) currentArc.preview( targetI );
						updateRot( );
						break;
					case 13: // Enter
					case 38: // Up
						if (isCloseUp == false) {
							doCloseUp( );
						}
						break;
					case 40: // Down
						if (isCloseUp == true) {
							closeCloseUp( );
						}
						break;
				}
			}
		}
		
	}	
}








