/**
 * @author Antti
 */
 package gfx.scene3d {
 	
 	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.text.*;
	import flash.geom.*;
	import flash.media.Sound;
	import flash.net.*;
	import flash.xml.*;
 	
 	import gfx.*;
 	import media.Song;
 	
 	import org.papervision3d.objects.*;
	import org.papervision3d.materials.*;
	
	import mx.effects.Tween;
 	import mx.effects.easing.*;
 	
	public class Album3D extends DisplayObject3D {
		
		// _________________________________________________________________________
		// CLASS CONSTANTS
				
		static public const COVER_LOADED	: String = "coverLoaded";
		
		// _________________________________________________________________________
		// SETTINGS
		
		protected var titleSpacing			: Number = 10; // Pixels between the title and the image
		protected var trackParser				: String = "http://www.anttikupila.com/parse-mp3.php?url="; // Parser to parse the mp3 file out of last.fm’s page
		protected var maxSongs				: int = 10; // per artist/album. Too many songs may mean they're blank -> won't play -> delay
		protected var scaleFactor				: Number = 2; // Scale imge up, so that text is higher resolution (less crappy when zoomed)
		
		// _________________________________________________________________________
		// VARIABLES
				
		protected var _artist					: String;
		protected var _image					: String;
		protected var _hasStream				: Boolean;
		protected var _match					: Number;
		protected var _mbid					: String;
		
		protected var imageLoader				: Loader;
		protected var imageData				: BitmapData;
		
		protected var title					: TextField;
		protected var plane					: Plane;
		
		protected var xmlLoader				: URLLoader;
		protected var songs					: Array;
		protected var currentSongIndex		: int;
		
		// _________________________________________________________________________
		// CONSTRUCTOR
		
		function Album3D( artist : String, image : String ) {
			_artist = artist;
			_image = image;
			_match = match;
			_mbid = mbid;
			
			imageLoader = new Loader( );
			imageLoader.contentLoaderInfo.addEventListener( Event.COMPLETE, imageLoaded );
			
			title = new TextField( );
			title.selectable = false;
			title.autoSize = TextFieldAutoSize.LEFT;
			title.antiAliasType = AntiAliasType.ADVANCED;
			
			var titleStyle : TextFormat = new TextFormat( );
				titleStyle.font = "myriad";
				titleStyle.color = 0xffffff;
				titleStyle.size = 12;
				
			title.defaultTextFormat = titleStyle;
			title.text = artist;
			
			xmlLoader = new URLLoader( );
			xmlLoader.addEventListener( Event.COMPLETE, xmlLoaded );
		}
				
		// EVENT HANDLERS
		protected function imageLoaded( e : Event ) : void {
			
			var loadedBmp : Bitmap = e.target.content as Bitmap;
			var bmp : BitmapData = loadedBmp.bitmapData;
			
			var w : int = Math.floor( title.textWidth );
				if ( w < bmp.width ) w = bmp.width;
			
			var bitmap : BitmapData = new BitmapData( w * scaleFactor, bmp.height * scaleFactor + title.textHeight + titleSpacing , true, 0x00000000 );
			var scaleMatrix : Matrix = new Matrix( );
				scaleMatrix.scale( scaleFactor, scaleFactor );
				bitmap.draw( title, scaleMatrix );
				
			var matrix : Matrix = new Matrix( );
				matrix.translate( 0, title.textHeight + titleSpacing );
				matrix.scale( scaleFactor, scaleFactor );
				
			bitmap.draw( bmp, matrix );
			bmp.dispose( );
			imageData = new ReflectedBitmapData( bitmap, 100, 25 );
			dispatchEvent( new Event( COVER_LOADED ) );
		}
		
		protected function xmlLoaded ( e: Event ) : void {
			songs = new Array( );
			
			var xml : XML = new XML ( xmlLoader.data );
			var c : int = 0;
			for each ( var i : XML in xml.track ) {
				c++;
				var song : Song = new Song ( _artist, i.name, i.url );
					song.addEventListener( Event.UNLOAD, songRemoveHandler );
				songs.push( song );
				if (c >= maxSongs) break;
			}
			
			if ( songs.length > 0 ) playNextTrack( null );
		}
		
		protected function playNextTrack( e : Event ) : void {
			if ( songs.length <= 0 ) {
				currentSongIndex = -1;
				return;
			}
			currentSongIndex = Math.floor( Math.random()*songs.length );
			songs[ currentSongIndex ].play( );
			songs[ currentSongIndex ].addEventListener( Event.SOUND_COMPLETE, playNextTrack );
		}
		
		protected function songRemoveHandler( e : Event ) : void {
			currentSongIndex = -1;
			e.target.removeEventListener( Event.SOUND_COMPLETE, playNextTrack );
			e.target.removeEventListener( Event.UNLOAD, songRemoveHandler );
			for (var i : Number = 0; i < songs.length; i++) {
				if ( songs[ i ] == e.target ) {
					songs.splice( i, 1 );
					break;
				}
			}
			playNextTrack( null );
		}
		
		// _________________________________________________________________________
		// PUBLIC API METHODS
		
		public function loadCover( ) : void {
			imageLoader.load( new URLRequest( /*http://www.anttikupila.com/crossdomain-proxy.php?url=" + */_image ) );
		}
		
		public function init( ) : void {
			var bitmapMaterial : BitmapMaterial = new BitmapMaterial( imageData, {fillColor: 0x111111, fillAlpha: 100} );
				bitmapMaterial.doubleSided = true;
			plane = new Plane( bitmapMaterial, imageData.width / scaleFactor, imageData.height / scaleFactor, 2, 2 );
			
			addChild( plane );
		}
		
		override public function toString( ) : String {
			return "Album: " + _artist;
		}
		
		public function preview( ) : void {
			if ( !songs ) {
				var regEx : RegExp = /[\ \/]/; 
				xmlLoader.load( new URLRequest( "http://ws.audioscrobbler.com/1.0/artist/" + _artist.replace( regEx, "+") + "/toptracks.xml" ) );
			} else {
				playNextTrack( null );
			}
		}
		
		public function stopPreview( ) : void {
			if ( currentSongIndex > -1 ) {
				songs[ currentSongIndex ].removeEventListener( Event.SOUND_COMPLETE, playNextTrack );
				songs[ currentSongIndex ].removeEventListener( Event.UNLOAD, songRemoveHandler );
				songs[ currentSongIndex ].close( );
			}
		}
		
		// _________________________________________________________________________
		// GET & SET METHODS
		
		public function get artist( ) : String {
			return _artist;
		}
		
		public function get image( ) : String {
			return _image;
		}
		
		public function get match( ) : Number {
			return _match;
		}
		
		public function get mbid( ) : String {
			return _mbid;
		}
		
	}
 }