/**
 * @author Antti
 */
 package gfx.scene3d {
 	
	import flash.xml.*;
	import flash.events.*;
 	
	import org.papervision3d.objects.*;
	import org.papervision3d.materials.*;
 	
	public class Arc extends DisplayObject3D {
		
		// _________________________________________________________________________
		// CLASS CONSTANTS
		
		static public const DEGREES		: Number = 135;
		
		// _________________________________________________________________________
		// SETTINGS
		
		protected var spacing				: Number = 25; // Px between albums, not quite correct 'cause perimeter is used instead of circumference
		protected var results				: int = 24; // Max 100
		
		// _________________________________________________________________________
		// VARIABLES
		
		protected var albumList			: Array = new Array( );
		protected var albumsToLoad		: Array = new Array( );
		protected var rootNode			: DisplayObject3D;
		
		// _________________________________________________________________________
		// CONSTRUCTOR
		
		function Arc( albumData : XML, position : Number, artist : String ) {
			
			rootNode = new DisplayObject3D( "Arc_" + position );
			addChild( rootNode );
			
			var index : int = 0;
			for each ( var i : XML in albumData.artist ) {
				if ( !checkIfAlreadyAdded( i.image ) ) {
					index++;
					var a : Album3D = new Album3D( i.name, i.image );
					albumsToLoad.push( a );
					if (index >= results) break;
				}
			}
			
			loadNextCover( );
		}
		
		// _________________________________________________________________________
		// PROTECTED METHODS
		
		protected function loadNextCover( ) : void {
			var nextAlbum : Album3D = albumsToLoad.shift( );
			if( nextAlbum ) {
				nextAlbum.addEventListener( Album3D.COVER_LOADED, coverLoadHandler );
				nextAlbum.loadCover( );
			}
		}
		
		protected function checkIfAlreadyAdded( imageUrl : String ) : Boolean {
			var added : Boolean = false;
			for each ( var i : Number in albumList ) {
				if ( albumList[ i ].image == imageUrl ) added = true;
			}
			return added;
		}
		
		protected function updatePositions( ) : void {
			var albumCount : int = albumList.length;
			var factor : Number = 1/(360/DEGREES);
			var perimeter : Number = albumCount * (160+spacing) / (DEGREES/360);
			var radius : Number = perimeter / Math.PI / 2;
			
			z = radius;
			
			for ( var i : Number = 0; i < albumList.length; i++ ) {
				var a : Album3D = albumList[ i ];
					a.z = Math.sin( i/(albumCount)*Math.PI*(DEGREES/180) ) * radius * -1;
					a.x = Math.cos( i/(albumCount)*Math.PI*(DEGREES/180) ) * radius * -1; // -1 -> start from left
					a.y = -70;
					a.lookAt( rootNode );
					a.pitch(10);
			}
			
			dispatchEvent( new Event( Event.CHANGE ) );
		}
		
		// _________________________________________________________________________
		// EVENT HANDLERS
		
		protected function coverLoadHandler( e : Event ) : void {
			var loadedCover : Album3D = e.target as Album3D;
				
			albumList.push( loadedCover );
			loadedCover.init( );
			updatePositions( );
						
			rootNode.addChild( loadedCover );
			loadNextCover( );
		}
		
		// _________________________________________________________________________
		// PUBLIC API METHODS
		
		public function preview( index : int ) : void {
			albumList[ index ].preview( );
		}
		
		public function stopPreview( index : int ) : void {
			albumList[ index ].stopPreview( );
		}
		
		// _________________________________________________________________________
		// GET & SET METHODS
		
		public function get count( ) : int {
			return albumList.length + albumsToLoad.length;
		}
 	}
	
 }